Attribute VB_Name = "modMain"
Option Explicit
'********************************************************************************************
' Written by Pranay Uppuluri
'     - PRUppL@aol.com
'     - http://members.xoom.com/win32realm (must be in lower case)
'
' A small Direct3DRM sample on Visual Basic 5.0 that rotates the 4 inner
' planets (Mercury, Venus, Earth, Mars) around the Sun. I might be off a lot
' by the speeds of them, their relative size, their color (I didn't put any
' textures on it, just applied closest color). So, please forgive my
' Astronomical Background.
' As to the real 3D look, this app is not that cool looking. I am still
' learning the Lighting effects.
' I DON'T PROVIDE ANY WARRANTY! USE IT AT YOUR OWN RISK!!!
'********************************************************************************************

' Some Windows API...
Declare Function ShowWindow Lib "user32" (ByVal hwnd As Long, ByVal nCmdShow As Long) As Long
Declare Function UpdateWindow Lib "user32" (ByVal hwnd As Long) As Long
Declare Function GetDC Lib "user32" (ByVal hwnd As Long) As Long
Declare Function GetDeviceCaps Lib "gdi32" (ByVal hdc As Long, ByVal nIndex As Long) As Long
Declare Function ReleaseDC Lib "user32" (ByVal hwnd As Long, ByVal hdc As Long) As Long
Declare Function GetClientRect Lib "user32" (ByVal hwnd As Long, lpRect As RECT) As Long
Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (pDst As Any, pSrc As Any, ByVal ByteLen As Long)

' GUID type
Public Type GUID
    Data1 As Long
    Data2 As Integer
    Data3 As Integer
    Data4(0 To 7) As Byte
End Type

' Light types...
Public Enum LIGHT_TYPES
    LIGHT_AMBIENT = D3DRMLIGHT_AMBIENT
    LIGHT_DIRECTIONAL = D3DRMLIGHT_DIRECTIONAL
    LIGHT_POINT = D3DRMLIGHT_POINT
    LIGHT_PARALLEL_POINT = D3DRMLIGHT_PARALLELPOINT
    LIGHT_SPOT = D3DRMLIGHT_SPOT
End Enum

' ShowWindow API Constants...
Public Const SW_HIDE = 0
Public Const SW_MAXIMIZE = 3
Public Const SW_MINIMIZE = 6
Public Const SW_NORMAL = 1
' GetDeviceCaps...
Public Const BITSPIXEL = 12

' Some public variables...
Public hMain        As Long                 ' Handle to main window (set at Form_Load first)
Public bEnd         As Boolean              ' True = App is ending
Public BPP          As Integer              ' Bits-per-pixel
Public lpD3DRM      As Direct3DRM           ' Main Direct3DRM object
Public Scene        As Direct3DRMFrame      ' Our main 3D World
Public Camera       As Direct3DRMFrame      ' Frame describing user's POV
Public Sun          As Direct3DRMFrame      ' Frame with the yellow sphere (the Sun)
Public Mercury      As Direct3DRMFrame      ' Frame with the planet Mercury
Public Venus        As Direct3DRMFrame      ' Frame with the planet Venus
Public Earth        As Direct3DRMFrame      ' Frame with the planet Earth
Public Mars         As Direct3DRMFrame      ' Frame with the planet Mars
Public LightFrame   As Direct3DRMFrame      ' Frame with the lights of the scene
Public lpDDClipper  As DirectDrawClipper    ' A clipper to draw to the screen
Public dev          As Direct3DRMDevice     ' Hardware on which we draw to
Public windev       As Direct3DRMWinDevice  ' Activate and Paint handler
Public view         As Direct3DRMViewPort   ' Window projecting 3D world
Public model        As D3DRMCOLORMODEL      ' Color model for this tutorial
Public bMinimized   As Boolean              ' True = Window is minimized

' Error Checking
Public Function ErrCheck()
    If UCase(Err.Description) = "DIVISION BY ZERO" Then Exit Function
    If Err.Number <> 0 Then Call MsgBox(MyErrToString(Err.Number), vbCritical, "Error"): _
                            Call FiniObjects
End Function

' Direct3DRM Initialization
Public Function D3DRM_Init() As Boolean
Dim lpDD As DirectDraw2
Dim lpD3D  As Direct3D

    On Error GoTo DXError
    
    ' See if the hMain is NOT 0; if it is, end the program
    If hMain = 0 Then MsgBox "The public variable hMain not set!", vbCritical, "Error": End
    
    ' See if Direct3D is supported on user's system
    Call DirectDrawCreate(ByVal 0&, lpDD, Nothing)  ' Create a DirectDraw object
    
    Set lpD3D = lpDD  ' In C: lpDD.QueryInterface(IID_IDirect3D, (void **), &lpD3D)
    
    ' Direct3D is supported on User's system.
    ' Create a main Direct3DRM object and a Device independent
    ' DirectDraw Clipper
    Call Direct3DRMCreate(lpD3DRM)
    
    Call DirectDrawCreateClipper(ByVal 0&, lpDDClipper, Nothing)
    Call lpDDClipper.SetHWnd(ByVal 0&, hMain)
    
    ' Release the DirectDraw and Direct3D objects
    Set lpDD = Nothing
    Set lpD3D = Nothing
    
    ' Set our D3DRMCOLORMODEL to RGB
    model = D3DCOLOR_RGB  ' TODO: Set your App's color model here
                          '       Ex: D3DCOLOR_MONO if you don't want
                          '           Color rendering
    
    ' Create the scene (just initialize it, DON'T RENDER!)
    Call CreateScene(Scene, Camera, dev, view)
    
    ' Show the main window and update it
    Call ShowWindow(hMain, SW_NORMAL)
    Call UpdateWindow(hMain)
    
    ' Now enter the Rendering Loop
    If bMinimized = False Then  ' If the window is minimized, don't render
        Do Until bEnd
            Call Scene.Move(1)      ' Move the scene
            Call view.Clear         ' Clear the viewport
            Call view.Render(Scene) ' Render the scene
            Call dev.Update         ' Update the window
            
            DoEvents ' Be nice
        Loop
    End If
    
    ' End the app since bEnd = True
    Call FiniObjects ' Finish objects
Exit Function
DXError:
        Call ErrCheck
        Resume
End Function

' Creates a dev, a Window dev and a view objects
Public Sub CreateDeviceAndView()
' *******************************************************************************************
Dim hTempDC As Long
Dim rc      As RECT
Dim lpGUID  As GUID  ' Pointer to the display driver's GUID

    On Error GoTo DXError
    
    ' First get the BPP on the user's system
    hTempDC = GetDC(0)
    BPP = GetDeviceCaps(hTempDC, BITSPIXEL)
    Call ReleaseDC(0, hTempDC)
    
    ' Create a dev that BEST suits our needs
    Call FindDevice(model, lpGUID)
    
    ' Now get the update region of the main window
    Call GetClientRect(hMain, rc)
    
    ' Try to create a Direct3DRMDevice using the Clipper
    Call lpD3DRM.CreateDeviceFromClipper(lpDDClipper, lpGUID, _
                                            rc.Right, rc.bottom, dev)
    
    ' Set the Windows Device to the D3DRMDevice
    Set windev = dev
    
    ' Now create a view and set its Back to 5000
    Call lpD3DRM.CreateViewport(dev, Camera, 0, 0, dev.GetWidth, dev.GetHeight, view)
    
    Call view.SetBack(5000!)  ' The farthest that the view will render
    
    ' Set the dev render quality to Gouraud shading with Lights on
    Call dev.SetQuality(D3DRMSHADE_GOURAUD Or D3DRMFILL_SOLID Or D3DRMLIGHT_ON)
   
    ' Set shade according to the bits per pixel.
    Select Case BPP
        Case 1:
            Call dev.SetShades(4)
            Call lpD3DRM.SetDefaultTextureShades(4)
        Case 16:
            Call dev.SetShades(32)
            Call lpD3DRM.SetDefaultTextureColors(64)
            Call lpD3DRM.SetDefaultTextureShades(32)
        Case 24:
        Case 32:
             Call dev.SetShades(256)
             Call lpD3DRM.SetDefaultTextureColors(64)
             Call lpD3DRM.SetDefaultTextureShades(256)
     End Select
     
     ' Don't put dither on
     Call dev.SetDither(False)
Exit Sub
DXError:
    Call ErrCheck
    Resume
End Sub

' Creates a new scene (DOESN'T RENDER IT!)
Public Function CreateScene(Scene As Direct3DRMFrame, Camera As Direct3DRMFrame, _
                            dev As Direct3DRMDevice, view As Direct3DRMViewPort) As Boolean
'********************************************************************************************
Dim sunPos      As D3DVECTOR

    On Error GoTo DXError
    
    ' Create a Scene, and add a camera to it
    Call lpD3DRM.CreateFrame(Nothing, Scene)  ' No parent frame = The world frame (scene)
    Call lpD3DRM.CreateFrame(Scene, Camera)
    
    ' Create additional frames
    Call lpD3DRM.CreateFrame(Scene, LightFrame)
    Call lpD3DRM.CreateFrame(Scene, Sun)
    Call lpD3DRM.CreateFrame(Sun, Mercury)
    Call lpD3DRM.CreateFrame(Sun, Venus)
    Call lpD3DRM.CreateFrame(Sun, Earth)
    Call lpD3DRM.CreateFrame(Sun, Mars)
    
    ' Create a Device, a Window device and a Viewport
    Call CreateDeviceAndView
    
    ' Now add lights to our scene
    Call CreateLight(LIGHT_AMBIENT, LightFrame)  ' An ambient light source
    Call CreateLight(LIGHT_POINT, Sun)  ' Sun is the point from where light is illuminated
    
    ' Set the LightFrame position on the scene
    Call LightFrame.SetPosition(Scene, 10!, 10!, 8!)
    
    ' Load the objects
    Call LoadPlanets(Sun, Mercury, Venus, Earth, Mars)
    
    ' And add the Light frame to the Scene and the sun
    Call Scene.AddChild(LightFrame)
    Call Scene.AddChild(Sun)
    
    ' Set the planet's position and rotation
    Call Sun.SetPosition(Scene, 0!, 0!, 30!)
    Call Sun.GetPosition(Scene, sunPos)  ' First, get the Sun's position
    
    ' Mercury
    Call Mercury.SetPosition(Scene, sunPos.X, sunPos.Y, sunPos.z - 25!)
    Call Mercury.SetRotation(Scene, 0!, 1!, 0!, 0.01!)
    
    ' Venus
    Call Venus.SetPosition(Scene, sunPos.X + 25, sunPos.Y, sunPos.z - 75!)
    Call Venus.SetRotation(Scene, 0!, 1!, 0!, 0.001!)
    
    ' Earth
    Call Earth.SetPosition(Scene, sunPos.X - 12, sunPos.Y, sunPos.z - 125!)
    Call Earth.SetRotation(Scene, 0!, 1!, 0!, 0.0001!)
    
    ' Mars
    Call Mars.SetPosition(Scene, sunPos.X + 100, sunPos.Y, sunPos.z - 175!)
    Call Mars.SetRotation(Scene, 0!, 1!, 0!, 0.00001!)
    
    ' Now, set the camera positions
    Call Camera.SetPosition(Scene, 0!, 0!, -250!)
    Call Camera.LookAt(Sun, Scene, D3DRMCONSTRAIN_Z)
    Call LightFrame.LookAt(Sun, Scene, D3DRMCONSTRAIN_Z)
    
    ' Return happy
    CreateScene = True
Exit Function
DXError:
    Call ErrCheck
    Resume
End Function

' Find a device (First try the hardware device)
Public Function FindDevice(model As D3DRMCOLORMODEL, lpGUID As GUID) As Boolean
Dim lpDD  As DirectDraw
Dim lpD3D As Direct3D

    On Error GoTo DXError
    
    ' Specify the characteristics of the search
    Dim devSearch As D3DFINDDEVICESEARCH     ' Identifies the dev found
    Static devResult As D3DFINDDEVICERESULT  ' The result
    
    ' Create the DirectDraw object and see if it supports
    ' Direct3D
    Call DirectDrawCreate(ByVal 0&, lpDD, Nothing)
        
    Set lpD3D = lpDD
        
    
    ' Now fill out the struct
    devSearch.dwSize = Len(devSearch)
    devSearch.dwFlags = D3DFDS_COLORMODEL
    devSearch.dcmColorModel = model
    
    ' Fill out search results struct
    devResult.dwSize = Len(devResult)
    
    ' The results of the call are returned in the hardware and
    ' software description slots of the D3DFINDDEVICERESULT struct
    Call lpD3D.FindDevice(devSearch, devResult)  ' Find the device
        
    
    ' We have found a device. See if it supports the current BPP
    ' rendering. If it doesn't, we need to go with the software
    ' rendering.
    If (devResult.ddHwDesc.dwFlags And Not (devResult.ddHwDesc.dwDeviceRenderBitDepth And BPPToDDBD(BPP))) Then
    
        devSearch.dwFlags = devSearch.dwFlags Or D3DFDS_HARDWARE
        devSearch.bHardware = False
        
        Dim devDummy As D3DFINDDEVICERESULT  ' Not used
        
        ' Clear the memory of devResult
        Call CopyMemory(devResult, devDummy, Len(devDummy))
        devResult.dwSize = Len(devResult)
        
        ' Redo the find with a software search
        Call lpD3D.FindDevice(devSearch, devResult)
            
    End If
    
    Set lpD3D = Nothing ' Release D3D object
    Set lpDD = Nothing  ' Release the DD object
        
    
    ' Return the GUID
    Call CopyMemory(lpGUID, devResult.GUID, Len(devResult.GUID))
    FindDevice = True
Exit Function
DXError:
    ErrCheck
    Resume
End Function

' BPPToDDBD
' Converts bits per pixel to a DirectDraw bit depth flag
Public Function BPPToDDBD(BPP As Integer) As Long
   Select Case BPP
      Case 1
         BPPToDDBD = DDBD_1
      Case 2
         BPPToDDBD = DDBD_2
      Case 4
         BPPToDDBD = DDBD_4
      Case 8
         BPPToDDBD = DDBD_8
      Case 16
         BPPToDDBD = DDBD_16
      Case 24
         BPPToDDBD = DDBD_24
      Case 32
         BPPToDDBD = DDBD_32
      Case Else
         BPPToDDBD = 0
   End Select
End Function

' Creates a light of the type Directional, Parallel Point,
' Spot, or Point and attatches it to the frame
Public Function CreateLight(ByVal LightType As LIGHT_TYPES, frame2 As Direct3DRMFrame) As Boolean
Dim light As Direct3DRMLight
Dim frame As Direct3DRMFrame

    On Error GoTo DXError
    
    ' See what type of light we are going to create
    Select Case LightType
        Case LIGHT_AMBIENT:      ' Ambient light
            Call lpD3DRM.CreateLightRGB(D3DRMLIGHT_AMBIENT, 0.1!, 0.1!, 0.1!, light)
        Case LIGHT_DIRECTIONAL:  ' Directional light
            ' Create a directional light
            Call lpD3DRM.CreateLightRGB(D3DRMLIGHT_DIRECTIONAL, 1!, 1!, 1!, light)
        Case LIGHT_PARALLEL_POINT: ' Parallel point
            Call lpD3DRM.CreateLightRGB(D3DRMLIGHT_PARALLELPOINT, 1!, 1!, 1!, light)
        Case LIGHT_SPOT:                 ' Spot light
            Call lpD3DRM.CreateLightRGB(D3DRMLIGHT_SPOT, 1!, 1!, 1!, light)
        Case LIGHT_POINT:               ' Point
            Call lpD3DRM.CreateLightRGB(D3DRMLIGHT_POINT, 1!, 1!, 0!, light)  ' Yellow light
    End Select
                    
    ' Create a frame for our objects
    Call lpD3DRM.CreateFrame(Scene, frame)
    Call frame.SetPosition(Camera, 0!, 0!, 10!)  ' Position it
    Call frame.AddLight(light)
    
    If Not frame2 Is Nothing Then
        Call frame2.AddChild(frame)
    End If
    
    Set frame = Nothing
    Set light = Nothing
    
    CreateLight = True
Exit Function
DXError:
    ErrCheck
    Resume
End Function

' Loads our spheres, sets their scale etc.
Public Sub LoadPlanets(frame1 As Direct3DRMFrame, frame2 As Direct3DRMFrame, frame3 As Direct3DRMFrame, _
                       frame4 As Direct3DRMFrame, frame5 As Direct3DRMFrame)
Dim obj_Builder As Direct3DRMMeshBuilder
Dim wrap        As Direct3DRMWrap

    On Error GoTo DXError
    
    ' Create a new mesh builder and load the file
    Call lpD3DRM.CreateMeshBuilder(obj_Builder)
    Call obj_Builder.Load(App.Path & "\sphere3.x", ByVal 0&, D3DRMLOAD_FROMFILE, ByVal 0&, ByVal 0&)
    Call obj_Builder.[Scale](5!, 5!, 5!)  ' Sun
    Call obj_Builder.SetColorRGB(1!, 1!, 0!) ' Yellow colored sphere
    Call obj_Builder.SetQuality(D3DRMSHADE_GOURAUD Or D3DRMFILL_SOLID)
    Call frame1.AddVisual(obj_Builder)  ' Add the Sun visual object
    Call frame1.SetRotation(Scene, 0!, 0.1!, 0!, 0.0159!)  ' THIS IS THE SPEED OF THE SCENE
    Set obj_Builder = Nothing  ' Release the obj_Builder
    
    ' Now load Mercury object
    Call lpD3DRM.CreateMeshBuilder(obj_Builder)
    Call obj_Builder.Load(App.Path & "\sphere3.x", ByVal 0&, D3DRMLOAD_FROMFILE, ByVal 0&, ByVal 0&)
    Call obj_Builder.[Scale](1!, 1!, 1!) ' Mercury
    Call obj_Builder.SetColorRGB(0.5!, 0.5!, 0.5!) ' Grey colored sphere
    Call obj_Builder.SetQuality(D3DRMSHADE_GOURAUD Or D3DRMFILL_SOLID)
    Call frame2.AddVisual(obj_Builder)  ' Add the Mercury visual object
    Set obj_Builder = Nothing  ' Release the obj_Builder
    
    ' Now load Venus object
    Call lpD3DRM.CreateMeshBuilder(obj_Builder)
    Call obj_Builder.Load(App.Path & "\sphere3.x", ByVal 0&, D3DRMLOAD_FROMFILE, ByVal 0&, ByVal 0&)
    Call obj_Builder.[Scale](1.7!, 1.7!, 1.7!) ' A little bit bigger than Mercury
    Call obj_Builder.SetColorRGB(0.9!, 0.9!, 0.8!) ' Bright (white) colored sphere
    Call obj_Builder.SetQuality(D3DRMSHADE_GOURAUD Or D3DRMFILL_SOLID)
    Call frame3.AddVisual(obj_Builder)  ' Add the Venus visual object
    Set obj_Builder = Nothing  ' Release the obj_Builder
    
    ' Now load the Earth
    Call lpD3DRM.CreateMeshBuilder(obj_Builder)
    Call obj_Builder.Load(App.Path & "\sphere3.x", ByVal 0&, D3DRMLOAD_FROMFILE, ByVal 0&, ByVal 0&)
    Call obj_Builder.[Scale](2!, 2!, 2!) ' Earth is slightly bigger than Venus
    Call obj_Builder.SetColorRGB(0!, 0!, 0.7!) ' Blue colored sphere
    Call obj_Builder.SetQuality(D3DRMSHADE_GOURAUD Or D3DRMFILL_SOLID)
    Call frame4.AddVisual(obj_Builder)  ' Add the Earth visual object
    Set obj_Builder = Nothing  ' Release the obj_Builder
    
    ' Mars
    Call lpD3DRM.CreateMeshBuilder(obj_Builder)
    Call obj_Builder.Load(App.Path & "\sphere3.x", ByVal 0&, D3DRMLOAD_FROMFILE, ByVal 0&, ByVal 0&)
    Call obj_Builder.[Scale](1.8!, 1.8!, 1.8!) ' Mars is a little bit smaller than Earth
    Call obj_Builder.SetColorRGB(0.5!, 0!, 0!) ' Red colored sphere
    Call obj_Builder.SetQuality(D3DRMSHADE_GOURAUD Or D3DRMFILL_SOLID)
    Call frame5.AddVisual(obj_Builder)  ' Add the Mars visual object
    Set obj_Builder = Nothing  ' Release the obj_Builder
Exit Sub
DXError:
    ErrCheck
    Resume
End Sub

' Ends the program; call this method BEFORE ENDING THE PROGRAM!
Public Sub FiniObjects()
    ' Ending = True
    bEnd = True
    
    ' Release all the DirectX object
    Set Sun = Nothing
    Set Mercury = Nothing
    Set view = Nothing
    Set windev = Nothing
    Set dev = Nothing
    Set Camera = Nothing
    Set Scene = Nothing
    Set lpD3DRM = Nothing
    
    ' End the program
    End
End Sub

' Gets the Error string specified the Error number
Public Function MyErrToString(ByVal errVal As Long) As String

    Select Case errVal
        ' DirectDraw errors
        Case DDERR_ALREADYINITIALIZED: MyErrToString = "DDERR_ALREADYINITIALIZED"
        Case DDERR_BLTFASTCANTCLIP: MyErrToString = "DDERR_BLTFASTCANTCLIP"
        Case DDERR_CANNOTATTACHSURFACE: MyErrToString = "DDERR_CANNOTATTACHSURFACE"
        Case DDERR_CANTCREATEDC: MyErrToString = "DDERR_CANTCREATEDC"
        Case DDERR_CANTDUPLICATE: MyErrToString = "DDERR_CANTDUPLICATE"
        Case DDERR_CANTLOCKSURFACE: MyErrToString = "DDERR_CANTLOCKSURFACE"
        Case DDERR_CANTPAGELOCK: MyErrToString = "DDERR_CANTPAGELOCK"
        Case DDERR_CANTPAGEUNLOCK: MyErrToString = "DDERR_CANTPAGEUNLOCK"
        Case DDERR_CLIPPERISUSINGHWND: MyErrToString = "DDERR_CLIPPERISUSINGHWND"
        Case DDERR_COLORKEYNOTSET: MyErrToString = "DDERR_COLORKEYNOTSET"
        Case DDERR_CURRENTLYNOTAVAIL: MyErrToString = "DDERR_CURRENTLYNOTAVAIL"
        Case DDERR_DCALREADYCREATED: MyErrToString = "DDERR_DCALREADYCREATED"
        Case DDERR_DEVICEDOESNTOWNSURFACE: MyErrToString = "DDERR_DEVICEDOESNTOWNSURFACE"
        Case DDERR_DIRECTDRAWALREADYCREATED: MyErrToString = "DDERR_DIRECTDRAWALREADYCREATED"
        Case DDERR_EXCEPTION: MyErrToString = "DDERR_EXCEPTION"
        Case DDERR_EXCLUSIVEMODEALREADYSET: MyErrToString = "DDERR_EXCLUSIVEMODEALREADYSET"
        'Case DDERR_GENERIC: MyErrToString = "DDERR_GENERIC"
        Case DDERR_HEIGHTALIGN: MyErrToString = "DDERR_HEIGHTALIGN"
        Case DDERR_HWNDALREADYSET: MyErrToString = "DDERR_HWNDALREADYSET"
        Case DDERR_HWNDSUBCLASSED: MyErrToString = "DDERR_HWNDSUBCLASSED"
        Case DDERR_IMPLICITLYCREATED: MyErrToString = "DERR_IMPLICITYCREATED"
        Case DDERR_INCOMPATIBLEPRIMARY: MyErrToString = "DERR_INCOMPATIBLEPRIMARY"
        Case DDERR_INVALIDCAPS: MyErrToString = "DDERR_INVALIDCAPS"
        Case DDERR_INVALIDCLIPLIST: MyErrToString = "DERR_INVALIDCLIPLIST"
        Case DDERR_INVALIDDIRECTDRAWGUID: MyErrToString = "DDERR_INVALIDDIRECTDRAWGUID"
        Case DDERR_INVALIDMODE: MyErrToString = "DERR_INVALIDMODE"
        Case DDERR_INVALIDOBJECT: MyErrToString = "DDERR_INVALIDOBJECT"
        'Case DDERR_INVALIDPARAMS: MyErrToString = "DDERR_INVALIDPARAMS"
        Case DDERR_INVALIDPIXELFORMAT: MyErrToString = "DDERR_INVALIDPIXELFORMAT"
        Case DDERR_INVALIDPOSITION: MyErrToString = "DDERR_INVALIDPOSITION"
        Case DDERR_INVALIDRECT: MyErrToString = "DDERR_INVALIDRECT"
        Case DDERR_INVALIDSURFACETYPE: MyErrToString = "DDERR_INVALIDSURFACETYPE"
        Case DDERR_LOCKEDSURFACES: MyErrToString = "DDERR_LOCKEDSURFACES"
        Case DDERR_MOREDATA: MyErrToString = "DDERR_MOREDATA"
        Case DDERR_NO3D: MyErrToString = "DDERR_NO3D"
        Case DDERR_NOALPHAHW: MyErrToString = "DDERR_NOALPHAHW"
        Case DDERR_NOBLTHW: MyErrToString = "DDERR_NOBLTHW"
        Case DDERR_NOCLIPLIST: MyErrToString = "DDERR_NOCLIPLIST"
        Case DDERR_NOCLIPPERATTACHED: MyErrToString = "DDERR_NOCLIPPERATTACHED"
        Case DDERR_NOCOLORCONVHW: MyErrToString = "DDERR_NOCOLORCONVHW"
        Case DDERR_NOCOLORKEY: MyErrToString = "DDERR_NOCOLORKEY"
        Case DDERR_NOCOLORKEYHW: MyErrToString = "DDERR_NOCOLORKEYHW"
        Case DDERR_NOCOOPERATIVELEVELSET: MyErrToString = "DDERR_NOCOOPERATIVELEVELSET"
        Case DDERR_NODC: MyErrToString = "DDERR_NODC"
        Case DDERR_NODDROPSHW: MyErrToString = "DDERR_NODDROPSHW"
        Case DDERR_NODIRECTDRAWHW: MyErrToString = "DDERR_NODIRECTDRAWHW"
        Case DDERR_NODIRECTDRAWSUPPORT: MyErrToString = "DDERR_NODIRECTDRAWSUPPORT"
        Case DDERR_NOEMULATION: MyErrToString = "DDERR_NOEMULATION"
        Case DDERR_NOEXCLUSIVEMODE: MyErrToString = "DDERR_NOEXCLUSIVEMODE"
        Case DDERR_NOFLIPHW: MyErrToString = "DDERR_NOFLIPHW"
        Case DDERR_NOHWND: MyErrToString = "DDERR_NOHWND"
        Case DDERR_NOMIPMAPHW: MyErrToString = "DDERR_NOMIPMAPHW"
        Case DDERR_NOMIRRORHW: MyErrToString = "DDERR_NOMIRRORHW"
        Case DDERR_NONONLOCALVIDMEM: MyErrToString = "DDERR_NONONLOCALVIDMEM"
        Case DDERR_NOOPTIMIZEHW: MyErrToString = "DDERR_NOOPTIMIZEHW"
        Case DDERR_NOOVERLAYDEST: MyErrToString = "DDERR_NOOVERLAYDEST"
        Case DDERR_NOOVERLAYHW: MyErrToString = "DDERR_NOOVERLAYHW"
        Case DDERR_NOPALETTEATTACHED: MyErrToString = "DDERR_NOPALETTEATTACHED"
        Case DDERR_NOPALETTEHW: MyErrToString = "DDERR_NOPALETTEHW"
        Case DDERR_NORASTEROPHW: MyErrToString = "DDERR_NORASTEROPHW"
        Case DDERR_NOROTATIONHW: MyErrToString = "DDERR_NOROTATIONHW"
        Case DDERR_NOSTRETCHHW: MyErrToString = "DDERR_NOSTRETCHHW"
        Case DDERR_NOT4BITCOLOR: MyErrToString = "DDERR_NOT4BITCOLOR"
        Case DDERR_NOT4BITCOLORINDEX: MyErrToString = "DDERR_NOT4BITCOLORINDEX"
        Case DDERR_NOT8BITCOLOR: MyErrToString = "DDERR_NOT8BITCOLOR"
        Case DDERR_NOTAOVERLAYSURFACE: MyErrToString = "DDERR_NOTAOVERLAYSURFACE"
        Case DDERR_NOTEXTUREHW: MyErrToString = "DDERR_NOTEXTUREHW"
        Case DDERR_NOTFLIPPABLE: MyErrToString = "DDERR_NOTFLIPPABLE"
        Case DDERR_NOTFOUND: MyErrToString = "DDERR_NOTFOUND"
        'Case DDERR_NOTINITIALIZED: MyErrToString = "DDERR_NOTINITIALIZED"
        Case DDERR_NOTLOADED: MyErrToString = "DDERR_NOTLOADED"
        Case DDERR_NOTLOCKED: MyErrToString = "DDERR_NOTLOCKED"
        Case DDERR_NOTPAGELOCKED: MyErrToString = "DDERR_NOTPAGELOCKED"
        Case DDERR_NOTPALETTIZED: MyErrToString = "DDERR_NOTPALETTIZED"
        Case DDERR_NOVSYNCHW: MyErrToString = "DDERR_NOVSYNCHW"
        Case DDERR_NOZBUFFERHW: MyErrToString = "DDERR_NOZBUFFERHW"
        Case DDERR_NOZOVERLAYHW: MyErrToString = "DDERR_NOZOVERLAYHW"
        Case DDERR_OUTOFCAPS: MyErrToString = "DDERR_OUTOFCAPS"
        'Case DDERR_OUTOFMEMORY: MyErrToString = "DDERR_OUTOFMEMORY"
        Case DDERR_OUTOFVIDEOMEMORY: MyErrToString = "DDERR_OUTOFVIDEOMEMORY"
        Case DDERR_OVERLAYCANTCLIP: MyErrToString = "DDERR_OVERLAYCANTCLIP"
        Case DDERR_OVERLAYCOLORKEYONLYONEACTIVE: MyErrToString = "DDERR_OVERLAYCOLORKEYONLYONEACTIVE"
        Case DDERR_OVERLAYNOTVISIBLE: MyErrToString = "DDERR_OVERLAYNOTVISIBLE"
        Case DDERR_PALETTEBUSY: MyErrToString = "DDERR_PALETTEBUSY"
        Case DDERR_PRIMARYSURFACEALREADYEXISTS: MyErrToString = "DDERR_PRIMARYSURFACEALREADYEXISTS"
        Case DDERR_REGIONTOOSMALL: MyErrToString = "DDERR_REGIONTOOSMALL"
        Case DDERR_SURFACEALREADYATTACHED: MyErrToString = "DDERR_SURFACEALREADYATTACHED"
        Case DDERR_SURFACEALREADYDEPENDENT: MyErrToString = "DDERR_SURFACEALREADYDEPENDENT"
        Case DDERR_SURFACEBUSY: MyErrToString = "DDERR_SURFACEBUSY"
        Case DDERR_SURFACEISOBSCURED: MyErrToString = "DDERR_SURFACEISOBSCURED"
        Case DDERR_SURFACELOST: MyErrToString = "DDERR_SURFACELOST"
        Case DDERR_SURFACENOTATTACHED: MyErrToString = "DDERR_SURFACENOTATTACHED"
        Case DDERR_TOOBIGHEIGHT: MyErrToString = "DDERR_TOOBIGHEIGHT"
        Case DDERR_TOOBIGSIZE: MyErrToString = "DDERR_TOOBIGSIZE"
        Case DDERR_TOOBIGWIDTH: MyErrToString = "DDERR_TOOBIGWIDTH"
        'Case DDERR_UNSUPPORTED: MyErrToString = "DDERR_UNSUPPORTED"
        Case DDERR_UNSUPPORTEDFORMAT: MyErrToString = "DDERR_UNSUPPORTEDFORMAT"
        Case DDERR_UNSUPPORTEDMASK: MyErrToString = "DDERR_UNSUPPORTEDMASK"
        Case DDERR_UNSUPPORTEDMODE: MyErrToString = "DDERR_UNSUPPORTEDMODE"
        Case DDERR_VERTICALBLANKINPROGRESS: MyErrToString = "DDERR_VERTICALBLANKINPROGRESS"
        Case DDERR_VIDEONOTACTIVE: MyErrToString = "DDERR_VIDEONOTACTIVE"
        Case DDERR_WASSTILLDRAWING: MyErrToString = "DDERR_WASSTILLDRAWING"
        Case DDERR_WRONGMODE: MyErrToString = "DDERR_WRONGMODE"
        Case DDERR_XALIGN: MyErrToString = "DDERR_XALIGN"
        ' Direct3D
        Case D3DRMERR_BADALLOC: MyErrToString = "D3DRMERR_BADALLOC"
        Case D3DRMERR_BADDEVICE: MyErrToString = "D3DRMERR_BADDEVICE"
        Case D3DRMERR_BADFILE: MyErrToString = "D3DRMERR_BADFILE"
        Case D3DRMERR_BADMAJORVERSION: MyErrToString = "D3DRMERR_BADMAJORVERSION"
        Case D3DRMERR_BADMINORVERSION: MyErrToString = "D3DRMERR_BADMINORVERSION"
        Case D3DRMERR_BADOBJECT: MyErrToString = "D3DRMERR_BADOBJECT"
        Case D3DRMERR_BADPMDATA: MyErrToString = "D3DRMERR_BADPMDATA"
        Case D3DRMERR_BADTYPE: MyErrToString = "D3DRMERR_BADTYPE"
        Case D3DRMERR_BADVALUE: MyErrToString = "D3DRMERR_BADVALUE"
        Case D3DRMERR_BOXNOTSET: MyErrToString = "D3DRMERR_BOXNOTSET"
        Case D3DRMERR_CONNECTIONLOST: MyErrToString = "D3DRMERR_CONNECTIONLOST"
        Case D3DRMERR_FACEUSED: MyErrToString = "D3DRMERR_FACEUSED"
        Case D3DRMERR_FILENOTFOUND: MyErrToString = "D3DRMERR_FILENOTFOUND"
        'Case D3DRMERR_INVALIDDATA: MyErrToString = "D3DRMERR_INVALIDDATA"
        'Case D3DRMERR_INVALIDOBJECT: MyErrToString = "D3DRMERR_INVALIDOBJECT"
        'Case D3DRMERR_INVALIDPARAMS: MyErrToString = "D3DRMERR_INVALIDPARAMS"
        Case D3DRMERR_NOTDONEYET: MyErrToString = "D3DRMERR_NOTDONEYET"
        Case D3DRMERR_NOTENOUGHDATA: MyErrToString = "D3DRMERR_NOTENOUGHDATA"
        Case D3DRMERR_NOTFOUND: MyErrToString = "D3DRMERR_NOTFOUND"
        Case D3DRMERR_PENDING: MyErrToString = "D3DRMERR_PENDING"
        Case D3DRMERR_REQUESTTOOLARGE: MyErrToString = "D3DRMERR_REQUESTTOOLARGE"
        Case D3DRMERR_REQUESTTOOSMALL: MyErrToString = "D3DRMERR_REQUESTTOOSMALL"
        Case D3DRMERR_UNABLETOEXECUTE: MyErrToString = "D3DRMERR_UNABLETOEXECUTE"
        Case Else: MyErrToString = "Error number is: " & Err.Number & vbCrLf & Err.Description
    End Select
End Function
